@@ -46,7 +46,6 @@ import org.apache.wicket.request.mapper.parameter.PageParameters;
 import org.apache.wicket.request.resource.IResource;
 import org.apache.wicket.request.resource.ResourceReference;
 import org.apache.wicket.request.resource.caching.IStaticCacheableResource;
-import org.apache.wicket.settings.IApplicationSettings;
 import org.apache.wicket.util.lang.Args;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -196,7 +195,12 @@ public class RequestCycle implements IRequestCycle, IEventSink
 	 */
 	protected int getExceptionRetryCount()
 	{
-		return 10;
+		int retries = 10;
+		if (Application.exists())
+		{
+			retries = Application.get().getRequestCycleSettings().getExceptionRetryCount();
+		}
+		return retries;
 	}
 
 	/**
@@ -390,9 +394,10 @@ public class RequestCycle implements IRequestCycle, IEventSink
 	 * @throws IllegalArgumentException
 	 * @see MetaDataKey
 	 */
-	public final <T> void setMetaData(final MetaDataKey<T> key, final T object)
+	public final <T> RequestCycle setMetaData(final MetaDataKey<T> key, final T object)
 	{
 		metaData = key.set(metaData, object);
+		return this;
 	}
 
 	/**
@@ -635,6 +640,10 @@ public class RequestCycle implements IRequestCycle, IEventSink
 		{
 			requestHandlerExecutor.detach();
 		}
+		catch (RuntimeException exception)
+		{
+			handleDetachException(exception);
+		}
 		finally
 		{
 			listeners.onDetach(this);
@@ -647,6 +656,32 @@ public class RequestCycle implements IRequestCycle, IEventSink
 
 	}
 
+	/**
+	 * Called to handle a {@link java.lang.RuntimeException} that might be 
+	 * thrown during detaching phase. 
+	 * 
+	 * @param exception
+	 */
+	private void handleDetachException(RuntimeException exception) 
+	{
+		boolean isBufferedResponse = true;
+		if (Application.exists())
+		{
+			isBufferedResponse = Application.get().getRequestCycleSettings().getBufferResponse();
+		}
+
+		//if application is using a buffered response strategy,
+		//then we display exception to user.
+		if (isBufferedResponse) 
+		{
+			throw exception;
+		}
+		else 
+		{
+			log.error("Error detaching RequestCycle", exception);
+		}
+	}
+
 	/**
 	 * Convenience method for setting next page to be rendered.
 	 * 
@@ -719,36 +754,6 @@ public class RequestCycle implements IRequestCycle, IEventSink
 				redirectPolicy));
 	}
 
-	/**
-	 * Gets whether or not feedback messages are to be cleaned up on detach.
-	 * 
-	 * @return true if they are
-	 * @deprecated see {@link IApplicationSettings#getFeedbackMessageCleanupFilter()}
-	 * 
-	 *             TODO 7.0 remove
-	 */
-	@Deprecated
-	public final boolean isCleanupFeedbackMessagesOnDetach()
-	{
-		throw new UnsupportedOperationException("Deprecated, see javadoc");
-	}
-
-	/**
-	 * Sets whether or not feedback messages should be cleaned up on detach.
-	 * 
-	 * @param cleanupFeedbackMessagesOnDetach
-	 *            true if you want them to be cleaned up
-	 * 
-	 * @deprecated see {@link #isCleanupFeedbackMessagesOnDetach()}
-	 * 
-	 *             TODO 7.0 remove
-	 */
-	@Deprecated
-	public final void setCleanupFeedbackMessagesOnDetach(boolean cleanupFeedbackMessagesOnDetach)
-	{
-		throw new UnsupportedOperationException("Deprecated, see javadoc");
-	}
-
 	/**
 	 * @return The start time for this request
 	 */
