@@ -18,6 +18,7 @@ package org.apache.wicket.request;
 
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.CoreMatchers.nullValue;
+import static org.junit.Assert.*;
 
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
@@ -31,14 +32,10 @@ import java.util.List;
 
 import org.apache.wicket.request.Url.QueryParameter;
 import org.apache.wicket.request.Url.StringMode;
-import org.junit.Assert;
 import org.junit.Test;
 
-/**
- * @author Matej Knopp
- * @author Igor Vaynberg
- */
-public class UrlTest extends Assert
+@SuppressWarnings("javadoc")
+public class UrlTest
 {
 	private void checkSegments(Url url, String... segments)
 	{
@@ -47,7 +44,7 @@ public class UrlTest extends Assert
 
 	private void checkQueryParams(Url url, String... params)
 	{
-		List<QueryParameter> list = new ArrayList<QueryParameter>();
+		List<QueryParameter> list = new ArrayList<>();
 		for (int i = 0; i < params.length; i += 2)
 		{
 			QueryParameter p = new QueryParameter(params[i], params[i + 1]);
@@ -57,9 +54,6 @@ public class UrlTest extends Assert
 		assertEquals(list, url.getQueryParameters());
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void parse1()
 	{
@@ -70,9 +64,6 @@ public class UrlTest extends Assert
 		assertEquals("foo2", url.getFragment());
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void parse2()
 	{
@@ -82,9 +73,6 @@ public class UrlTest extends Assert
 		checkQueryParams(url, "", "4", "6", "");
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void parse3()
 	{
@@ -97,22 +85,16 @@ public class UrlTest extends Assert
 		checkQueryParams(url);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void parse4()
 	{
 		String s = "/foo/bar//";
 		Url url = Url.parse(s);
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isContextAbsolute());
 		checkSegments(url, "", "foo", "bar", "", "");
 		checkQueryParams(url);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void parse5()
 	{
@@ -123,8 +105,18 @@ public class UrlTest extends Assert
 	}
 
 	/**
-	 * 
+	 * Same as #parse5() but with full url and not encoded '=' char in the query string
+	 * WICKET-5157
 	 */
+	@Test
+	public void parse5_1()
+	{
+		String s = "http://host:12345/foo/b%3Dr/b%26z/x%3F?a=b&x%3F%264=y=z";
+		Url url = Url.parse(s);
+		checkSegments(url, "", "foo", "b=r", "b&z", "x?");
+		checkQueryParams(url, "a", "b", "x?&4", "y=z");
+	}
+
 	@Test
 	public void parse6()
 	{
@@ -134,9 +126,6 @@ public class UrlTest extends Assert
 		checkQueryParams(url);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void parse7()
 	{
@@ -146,67 +135,52 @@ public class UrlTest extends Assert
 		checkQueryParams(url, "a", "b");
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void parse8()
 	{
 		String s = "/";
 		Url url = Url.parse(s);
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isContextAbsolute());
 		checkSegments(url, "", "");
 		checkQueryParams(url);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void parse9()
 	{
 		String s = "/?a=b";
 		Url url = Url.parse(s);
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isContextAbsolute());
 		checkSegments(url, "", "");
 		checkQueryParams(url, "a", "b");
 	}
 
-	/**
-	 *
-	 */
 	@Test
 	public void parse10()
 	{
 		String s = "/?a";
 		Url url = Url.parse(s);
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isContextAbsolute());
 		checkSegments(url, "", "");
 		checkQueryParams(url, "a", "");
 	}
 
-	/**
-	 *
-	 */
 	@Test
 	public void parse11()
 	{
 		String s = "/?a=";
 		Url url = Url.parse(s);
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isContextAbsolute());
 		checkSegments(url, "", "");
 		checkQueryParams(url, "a", "");
 	}
 
-	/**
-	 *
-	 */
 	@Test
 	public void parse12()
 	{
 		String s = "/?=b";
 		Url url = Url.parse(s);
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isContextAbsolute());
 		checkSegments(url, "", "");
 		checkQueryParams(url, "", "b");
 	}
@@ -219,7 +193,7 @@ public class UrlTest extends Assert
 	{
 		String s = "/?a=b&";
 		Url url = Url.parse(s);
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isContextAbsolute());
 		checkSegments(url, "", "");
 		checkQueryParams(url, "a", "b");
 	}
@@ -232,7 +206,7 @@ public class UrlTest extends Assert
 	{
 		String s = "/?a=b&+";
 		Url url = Url.parse(s);
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isContextAbsolute());
 		checkSegments(url, "", "");
 		checkQueryParams(url, "a", "b", " ", "");
 	}
@@ -322,9 +296,6 @@ public class UrlTest extends Assert
 		assertThat(url.getFragment(), is(nullValue()));
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void render1()
 	{
@@ -339,9 +310,6 @@ public class UrlTest extends Assert
 		assertEquals("foo/b=r/b&z/x%3F?a=b&x%3F%264=y%3Dz", url.toString());
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void render2()
 	{
@@ -350,9 +318,6 @@ public class UrlTest extends Assert
 		assertEquals(url.toString(), s);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void render3()
 	{
@@ -361,9 +326,6 @@ public class UrlTest extends Assert
 		assertEquals(url.toString(StringMode.FULL), s);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void render4()
 	{
@@ -399,60 +361,47 @@ public class UrlTest extends Assert
 		assertEquals("https://www.domain.com/foo/bar?baz=ban#bat", url.toString(StringMode.FULL));
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void absolute1()
 	{
 		Url url = Url.parse("abc/efg");
-		assertFalse(url.isAbsolute());
+		assertFalse(url.isFull());
+		assertFalse(url.isContextAbsolute());
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void absolute2()
 	{
 		Url url = Url.parse("");
-		assertFalse(url.isAbsolute());
+		assertFalse(url.isFull());
+		assertFalse(url.isContextAbsolute());
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void absolute3()
 	{
 		Url url = Url.parse("/");
-		assertTrue(url.isAbsolute());
+		assertFalse(url.isFull());
+		assertTrue(url.isContextAbsolute());
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void absolute4()
 	{
 		Url url = Url.parse("/abc/efg");
-		assertTrue(url.isAbsolute());
+		assertFalse(url.isFull());
+		assertTrue(url.isContextAbsolute());
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void absolute5()
 	{
 		Url url = Url.parse("http://www.domain.com");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isFull());
+		assertFalse(url.isContextAbsolute());
 	}
 
 
-	/**
-	 * 
-	 */
 	@Test
 	public void concat1()
 	{
@@ -461,9 +410,6 @@ public class UrlTest extends Assert
 		assertEquals(Url.parse("abc/xx/yy"), url);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void concat2()
 	{
@@ -472,9 +418,6 @@ public class UrlTest extends Assert
 		assertEquals(Url.parse("xx/yy"), url);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void concat3()
 	{
@@ -483,9 +426,6 @@ public class UrlTest extends Assert
 		assertEquals(Url.parse("xx/yy"), url);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void concat4()
 	{
@@ -494,9 +434,6 @@ public class UrlTest extends Assert
 		assertEquals(Url.parse("../xx/yy"), url);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void concat5()
 	{
@@ -505,9 +442,6 @@ public class UrlTest extends Assert
 		assertEquals(Url.parse("abc/efg/xx/yy"), url);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void concat6()
 	{
@@ -516,9 +450,6 @@ public class UrlTest extends Assert
 		assertEquals(Url.parse("abc/"), url);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void concat7()
 	{
@@ -527,9 +458,6 @@ public class UrlTest extends Assert
 		assertEquals(Url.parse(""), url);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void concat8()
 	{
@@ -538,9 +466,6 @@ public class UrlTest extends Assert
 		assertEquals(Url.parse("fff/abc/"), url);
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void concat9()
 	{
@@ -728,99 +653,93 @@ public class UrlTest extends Assert
 		return (Url)ois.readObject();
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void parseRelativeUrl()
 	{
 		Url url = Url.parse("foo");
 		checkUrl(url, null, null, null, "foo");
-		assertFalse(url.isAbsolute());
+		assertFalse(url.isContextAbsolute());
 
 		url = Url.parse("foo/bar/baz");
 		checkUrl(url, null, null, null, "foo", "bar", "baz");
-		assertFalse(url.isAbsolute());
+		assertFalse(url.isContextAbsolute());
 
 		url = Url.parse("?foobar");
 		checkUrl(url, null, null, null);
 		assertEquals("", url.getQueryParameter("foobar").getValue());
-		assertFalse(url.isAbsolute());
+		assertFalse(url.isContextAbsolute());
 
 		url = Url.parse("foo?a=123");
 		checkUrl(url, null, null, null, "foo");
 		assertEquals("123", url.getQueryParameter("a").getValue());
-		assertFalse(url.isAbsolute());
+		assertFalse(url.isContextAbsolute());
 
 		url = Url.parse("/foo");
 		checkUrl(url, null, null, null, "", "foo");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isContextAbsolute());
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void parseAbsoluteUrl()
 	{
 		Url url = Url.parse("ftp://myhost:8081");
 		checkUrl(url, "ftp", "myhost", 8081, "", "");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isFull());
 		assertEquals("ftp://myhost:8081/", url.toString(StringMode.FULL));
 
 		url = Url.parse("gopher://myhost:8081/foo");
 		checkUrl(url, "gopher", "myhost", 8081, "", "foo");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isFull());
 		assertEquals("gopher://myhost:8081/foo", url.toString(StringMode.FULL));
 
 		url = Url.parse("http://myhost:80/foo");
 		checkUrl(url, "http", "myhost", 80, "", "foo");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isFull());
 		assertEquals("http://myhost/foo", url.toString(StringMode.FULL));
 
 		url = Url.parse("http://myhost:81/foo");
 		checkUrl(url, "http", "myhost", 81, "", "foo");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isFull());
 		assertEquals("http://myhost:81/foo", url.toString(StringMode.FULL));
 
 		url = Url.parse("http://myhost/foo");
 		checkUrl(url, "http", "myhost", 80, "", "foo");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isFull());
 		assertEquals("http://myhost/foo", url.toString(StringMode.FULL));
 
 		url = Url.parse("https://myhost:443/foo");
 		checkUrl(url, "https", "myhost", 443, "", "foo");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isFull());
 		assertEquals("https://myhost/foo", url.toString(StringMode.FULL));
 
 		url = Url.parse("HTTPS://myhost/foo:123");
 		checkUrl(url, "https", "myhost", 443, "", "foo:123");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isFull());
 		assertEquals("https://myhost/foo:123", url.toString(StringMode.FULL));
 
 		url = Url.parse("ftp://myhost/foo");
 		checkUrl(url, "ftp", "myhost", 21, "", "foo");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isFull());
 		assertEquals("ftp://myhost/foo", url.toString(StringMode.FULL));
 
 		url = Url.parse("ftp://myhost:21/foo");
 		checkUrl(url, "ftp", "myhost", 21, "", "foo");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isFull());
 		assertEquals("ftp://myhost/foo", url.toString(StringMode.FULL));
 
 		url = Url.parse("ftp://user:pass@myhost:21/foo");
 		checkUrl(url, "ftp", "user:pass@myhost", 21, "", "foo");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isFull());
 		assertEquals("ftp://user:pass@myhost/foo", url.toString(StringMode.FULL));
 
 		url = Url.parse("FTp://myhost/foo");
 		checkUrl(url, "ftp", "myhost", 21, "", "foo");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isFull());
 		assertEquals("ftp://myhost/foo", url.toString(StringMode.FULL));
 
 		url = Url.parse("unknown://myhost/foo");
 		checkUrl(url, "unknown", "myhost", null, "", "foo");
-		assertTrue(url.isAbsolute());
+		assertTrue(url.isFull());
 		assertEquals("unknown://myhost/foo", url.toString(StringMode.FULL));
 	}
 
@@ -903,9 +822,6 @@ public class UrlTest extends Assert
 
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void prependLeadingSegments1()
 	{
@@ -916,9 +832,6 @@ public class UrlTest extends Assert
 		assertEquals("x/y/a", url.toString());
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void prependLeadingSegments2()
 	{
@@ -929,9 +842,6 @@ public class UrlTest extends Assert
 		assertEquals("x/a", url.toString());
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void prependLeadingSegments3()
 	{
@@ -942,9 +852,6 @@ public class UrlTest extends Assert
 		assertEquals("a", url.toString());
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void prependLeadingSegments4()
 	{
@@ -955,9 +862,6 @@ public class UrlTest extends Assert
 		assertEquals("x", url.toString());
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void removeLeadingSegments1()
 	{
@@ -967,9 +871,6 @@ public class UrlTest extends Assert
 		assertEquals("b", url.toString());
 	}
 
-	/**
-	 * 
-	 */
 	@Test
 	public void removeLeadingSegments2()
 	{
@@ -979,9 +880,6 @@ public class UrlTest extends Assert
 		assertEquals("", url.toString());
 	}
 
-	/**
-	 * 
-	 */
 	@Test(expected = IllegalArgumentException.class)
 	public void removeLeadingSegments3()
 	{
@@ -990,21 +888,39 @@ public class UrlTest extends Assert
 		url.removeLeadingSegments(3);
 	}
 
-    @Test
-    public void wicket_5114_allowtoStringFullWhenContainingTwoDots()
-    {
-        Url url = Url.parse("/mountPoint/whatever.../");
-        url.setHost("wicketHost");
-        assertEquals("//wicketHost/mountPoint/whatever.../", url.toString(StringMode.FULL));
-    }
+	@Test
+	public void wicket_5114_allowtoStringFullWhenContainingTwoDots()
+	{
+		Url url = Url.parse("/mountPoint/whatever.../");
+		url.setHost("wicketHost");
+		assertEquals("//wicketHost/mountPoint/whatever.../", url.toString(StringMode.FULL));
+	}
 
-    @Test(expected = IllegalStateException.class)
-    public void wicket_5114_throwExceptionWhenToStringFullContainsRelativePathSegment()
-    {
-        Url url = Url.parse("/mountPoint/../whatever/");
-        url.setHost("wicketHost");
-        url.toString(StringMode.FULL);
-    }
+	@Test(expected = IllegalStateException.class)
+	public void wicket_5114_throwExceptionWhenToStringFullContainsRelativePathSegment()
+	{
+		Url url = Url.parse("/mountPoint/../whatever/");
+		url.setHost("wicketHost");
+		url.toString(StringMode.FULL);
+	}
+
+	@Test
+	public void contextRelativeIsntReadAsContextAbsolute() throws Exception
+	{
+		Url url = Url.parse("/segment");
+		url.setContextRelative(true);
+		assertThat(url.isContextRelative(), is(true));
+		assertThat(url.isContextAbsolute(), is(false));
+	}
+
+	@Test
+	public void contextRelativeIsntReadAsFull() throws Exception
+	{
+		Url url = Url.parse("http://www.example.com/segment");
+		url.setContextRelative(true);
+		assertThat(url.isContextRelative(), is(true));
+		assertThat(url.isFull(), is(false));
+	}
 
 	@Test
 	public void isContextAbsolute()
@@ -1081,5 +997,4 @@ public class UrlTest extends Assert
 		checkUrl(url, "http", "[::1]", 1234, "", "myapp");
 		
 	}
-
 }
